@@ -57,6 +57,31 @@ interface, we don't call it config because there is a DAGConfig object in
 the Python API and this might cause confusion
 
 All other sections should represent valid DAG properties.
+
+
+Partials [Provisional name]
+---------------------------
+Partials were introduced to support a common ML use case: train and serving
+pipelines. If we describe a training pipeline as a DAG, the only difference
+with its serving counterpart are the tasks (can be more than one) at the
+beginning (train: get historical data, serve: get single data point) and the
+end (train: fit a model, serve: load a model and predict). Everything that
+happens in the middle should be the same to avoid training-serving skew. There
+are two specific use cases: batch and online.
+
+To define a batch serving pipeline we define three files: pipeline.yaml
+(train DAG), pipeline-serve (serve DAG), and pipeline-features.yaml (partial).
+The first two follow the DAGSpec schema and use
+``import_tasks_from: pipeline-features.yaml``. Training is done via
+``ploomber build --entry-point pipeline.yaml`` and serving via
+``ploomber build --entry-point pipeline-serve.yaml``.
+
+An online API has a different mechanism because the inference pipeline needs
+a Python API to interface with a web framework, rpc, or similar. This logic
+is implemented by OnlineDAG, which takes a partial definition
+(e.g. ``pipeline-features.yaml``) and builds an in-memory DAG that can make
+predictions using ``OnlineDAG().predict()``. See ``OnlineDAG`` documentation
+for details.
 """
 import os
 import yaml
@@ -67,17 +92,21 @@ from glob import iglob
 from itertools import chain
 import pprint
 
-from ploomber import products
-from ploomber import DAG, tasks, SourceLoader
-from ploomber.util.util import (load_dotted_path, find_file_recursively,
-                                call_with_dictionary, add_to_sys_path)
+from ploomber.dag.DAG import DAG
+from ploomber.placeholders.SourceLoader import SourceLoader
+from ploomber.util.util import call_with_dictionary, add_to_sys_path
+from ploomber.util import dotted_path
+from ploomber.util.default import entry_point
 from ploomber.spec.TaskSpec import TaskSpec, suffix2taskclass
 from ploomber.util import validate
+from ploomber.util import default
 from ploomber.dag.DAGConfiguration import DAGConfiguration
 from ploomber.exceptions import DAGSpecInitializationError
 from ploomber.env.EnvDict import EnvDict
-from ploomber.env.expand import expand_raw_dictionary
+from ploomber.env.expand import expand_raw_dictionary, expand_raw_dictionaries
 from ploomber.tasks import NotebookRunner
+from ploomber.validators.string import (validate_product_class_name,
+                                        validate_task_class_name)
 
 logger = logging.getLogger(__name__)
 pp = pprint.PrettyPrinter(indent=4)
@@ -107,8 +136,9 @@ class DAGSpec(MutableMapping):
         by the corresponding values in "env". A regular :py:mod:`ploomber.Env`
         object is created, see documentation for details. If None and
         data is a dict, no env is loaded. If None and loaded from a YAML spec,
-        an env.yaml file is loaded from the YAML spec parent folder, if it
-        exists
+        an env.yaml file is loaded (in the current working diirectory), if
+        it doesn't exist, it is loaded from the YAML spec parent folder, if it
+        exists. If none of these exist, no env is loaded.
 
     lazy_import : bool, optional
         Whether to import dotted paths to initialize PythonCallables with the
@@ -116,60 +146,151 @@ class DAGSpec(MutableMapping):
         with the dotted path, which means some verifications such as import
         statements in that function's module are delayed until the pipeline
         is executed. This also applies to placeholders loaded using a
-        SourceLoader, if a placeholder does not exist, it will return
-        None instead of raising an error
+        SourceLoader, if a placeholder exists, it will return the path
+        to it, instead of an initialized Placeholder object, if it doesn't,
+        it will return None instead of raising an error.
+
+    reload : bool, optional
+        Reloads modules before getting dotted paths. Has no effect if
+        lazy_import=True
     """
-    def __init__(self, data, env=None, lazy_import=False):
+
+    # NOTE: lazy_import is used where we need to initialized a a spec but don't
+    # plan on running it. One use case is when exporting to Argo or Airflow:
+    # we don't want to raise errors if some dependency is missing because
+    # it can happen that the environment exporting the dag does not have
+    # all the dependencies required to run it. The second use case is when
+    # running "ploomber scaffold", we want to use DAGSpec machinery to parse
+    # the yaml spec and the use such information to add the task in the
+    # appropriate place, this by construction, means that the spec as it is
+    # cannot be converted to a dag yet, since it has at least one task
+    # whose source does not exist
+    def __init__(self,
+                 data,
+                 env=None,
+                 lazy_import=False,
+                 reload=False,
+                 parent_path=None):
         if isinstance(data, (str, Path)):
-            path = data
+            if parent_path is not None:
+                raise ValueError('parent_path must be None when '
+                                 f'initializing {type(self).__name__} with '
+                                 'a path to a YAML spec')
+            # this is only used to display an error message with the path
+            # to the loaded file
+            path_for_errors = data
             # resolve the parent path to make sources and products unambiguous
             # even if the current working directory changes
-            self._parent_path = str(Path(data).parent.resolve())
+            path_to_entry_point = Path(data).resolve()
+            self._parent_path = str(path_to_entry_point.parent)
 
-            with open(str(data)) as f:
-                data = yaml.load(f, Loader=yaml.SafeLoader)
+            content = Path(data).read_text()
 
-            env_path = Path(self._parent_path, 'env.yaml')
+            try:
+                data = yaml.safe_load(content)
+            except (yaml.parser.ParserError,
+                    yaml.constructor.ConstructorError) as e:
+                error = e
+            else:
+                error = None
+
+            if error:
+                if '{{' in content or '}}' in content:
+                    raise DAGSpecInitializationError(
+                        'Failed to initialize spec. It looks like '
+                        'you\'re using placeholders (i.e. {{placeholder}}). '
+                        'Make sure values are enclosed in parentheses '
+                        '(e.g. key: "{{placeholder}}"). Original '
+                        'parser error:\n\n'
+                        f'{error}')
+                else:
+                    raise error
 
-            if env is None and env_path.exists():
-                env = str(env_path)
         else:
-            path = None
-            self._parent_path = None
+            path_for_errors = None
+            # FIXME: add test cases, some of those features wont work if
+            # _parent_path is None. We should make sure that we either raise
+            # an error if _parent_path is needed or use the current working
+            # directory if it's appropriate - this is mostly to make relative
+            # paths consistent: they should be relative to the file that
+            # contains them
+            self._parent_path = (None if not parent_path else str(
+                Path(parent_path).resolve()))
+
+        # try to look env.yaml in default locations
+        env_default_path = default.path_to_env(self._parent_path)
 
         self.data = data
 
         if isinstance(self.data, list):
-            self.data = data = {'tasks': self.data}
+            self.data = {'tasks': self.data}
 
         # validate keys defined at the top (nested keys are not validated here)
-        self._validate_top_keys(self.data, path)
+        self._validate_top_keys(self.data, path_for_errors)
 
         logger.debug('DAGSpec enviroment:\n%s', pp.pformat(env))
 
-        if env is not None:
-            # NOTE: when loading from a path, EnvDict recursively looks
-            # at parent folders, this is useful when loading envs
-            # in nested directories where scripts/functions need the env
-            # but here, since we just need this for the spec, we might
-            # want to turn it off. should we add a parameter to EnvDict
-            # to control this?
-            self.env = EnvDict(env)
-            self.data = expand_raw_dictionary(self.data, self.env)
+        env = env or dict()
+
+        # NOTE: when loading from a path, EnvDict recursively looks
+        # at parent folders, this is useful when loading envs
+        # in nested directories where scripts/functions need the env
+        # but here, since we just need this for the spec, we might
+        # want to turn it off. should we add a parameter to EnvDict
+        # to control this?
+        if env_default_path:
+            defaults = yaml.safe_load(Path(env_default_path).read_text())
+            self.env = EnvDict(env,
+                               path_to_here=self._parent_path,
+                               defaults=defaults)
         else:
-            self.env = None
+            self.env = EnvDict(env, path_to_here=self._parent_path)
+
+        self.data = expand_raw_dictionary(self.data, self.env)
 
         logger.debug('Expanded DAGSpec:\n%s', pp.pformat(data))
 
         # if there is a "location" top key, we don't have to do anything else
         # as we will just load the dotted path when .to_dag() is called
         if 'location' not in self.data:
+
+            Meta.initialize_inplace(self.data)
+
+            import_tasks_from = self.data['meta']['import_tasks_from']
+
+            if import_tasks_from is not None:
+                # when using a relative path in "import_tasks_from", we must
+                # make it absolute...
+                if not Path(import_tasks_from).is_absolute():
+                    # use _parent_path if there is one
+                    if self._parent_path:
+                        self.data['meta']['import_tasks_from'] = str(
+                            Path(self._parent_path, import_tasks_from))
+                    # otherwise just make it absolute
+                    else:
+                        self.data['meta']['import_tasks_from'] = str(
+                            Path(import_tasks_from).resolve())
+
+                imported = yaml.safe_load(
+                    Path(self.data['meta']['import_tasks_from']).read_text())
+
+                if self.env is not None:
+                    imported = expand_raw_dictionaries(imported, self.env)
+
+                # relative paths here are relative to the file where they
+                # are declared
+                base_path = Path(self.data['meta']['import_tasks_from']).parent
+
+                for task in imported:
+                    add_base_path_to_source_if_relative(task,
+                                                        base_path=base_path)
+
+                self.data['tasks'].extend(imported)
+
             self.data['tasks'] = [
                 normalize_task(task) for task in self.data['tasks']
             ]
 
-            Meta.initialize_inplace(self.data)
-
             # make sure the folder where the pipeline is located is in sys.path
             # otherwise dynamic imports needed by TaskSpec will fail
             with add_to_sys_path(self._parent_path, chdir=False):
@@ -177,8 +298,8 @@ class DAGSpec(MutableMapping):
                     TaskSpec(t,
                              self.data['meta'],
                              project_root=self._parent_path,
-                             lazy_import=lazy_import)
-                    for t in self.data['tasks']
+                             lazy_import=lazy_import,
+                             reload=reload) for t in self.data['tasks']
                 ]
         else:
             self.data['meta'] = Meta.empty()
@@ -196,7 +317,10 @@ class DAGSpec(MutableMapping):
                 raise KeyError('If specifying dag through a "location" key '
                                'it must be the unique key in the spec')
         else:
-            valid = {'meta', 'config', 'clients', 'tasks'}
+            valid = {
+                'meta', 'config', 'clients', 'tasks', 'serializer',
+                'unserializer'
+            }
             validate.keys(valid, spec.keys(), name='dag spec')
 
     def __getitem__(self, key):
@@ -237,10 +361,7 @@ class DAGSpec(MutableMapping):
         object
         """
         if 'location' in self:
-            factory = load_dotted_path(self['location'])
-            return factory()
-
-        tasks = self.pop('tasks')
+            return dotted_path.call_dotted_path(self['location'])
 
         dag = DAG()
 
@@ -250,19 +371,30 @@ class DAGSpec(MutableMapping):
         clients = self.get('clients')
 
         if clients:
-            init_clients(dag, clients)
+            for class_name, dotted_path_spec in clients.items():
+                dag.clients[class_name] = dotted_path.call_spec(
+                    dotted_path_spec)
 
-        process_tasks(dag, tasks, self, root_path=self._parent_path)
+        # FIXME: this violates lazy_import, we must change DAG's implementation
+        # to accept strings as attribute and load them until they are called
+        for attr in ['serializer', 'unserializer']:
+            if attr in self:
+                setattr(dag, attr, dotted_path.load_dotted_path(self[attr]))
+
+        process_tasks(dag, self, root_path=self._parent_path)
 
         return dag
 
     @classmethod
-    def auto_load(cls,
-                  to_dag=True,
-                  starting_dir=None,
-                  env=None,
-                  lazy_import=False):
+    def _auto_load(cls,
+                   to_dag=True,
+                   starting_dir=None,
+                   env=None,
+                   lazy_import=False,
+                   reload=False):
         """
+        NOTE: this is a private API. Use DAGSpec.find() instead
+
         Looks for a pipeline.yaml, generates a DAGSpec and returns a DAG.
         Currently, this is only used by the PloomberContentsManager, this is
         not intended to be a public API since initializing specs from paths
@@ -276,17 +408,19 @@ class DAGSpec(MutableMapping):
 
         Returns DAG and the directory where the pipeline.yaml file is located.
         """
-        path = find_file_recursively('pipeline.yaml',
-                                     starting_dir=starting_dir or os.getcwd())
+        root_path = starting_dir or os.getcwd()
+        rel_path = entry_point(root_path=root_path)
 
-        if path is None:
+        if rel_path is None:
             if to_dag:
                 return None, None, None
             else:
                 return None, None
 
+        path = Path(root_path, rel_path)
+
         try:
-            spec = cls(path, env=env, lazy_import=lazy_import)
+            spec = cls(path, env=env, lazy_import=lazy_import, reload=reload)
 
             if to_dag:
                 return spec, spec.to_dag(), Path(path).parent
@@ -298,9 +432,29 @@ class DAGSpec(MutableMapping):
                                              'pipeline.yaml')
             raise exc from e
 
+    @classmethod
+    def find(cls, env=None, reload=False):
+        """
+        Automatically find pipeline.yaml and return a DAGSpec object, which
+        can be converted to a DAG using .to_dag()
+
+        Parameters
+        ----------
+        env
+            The environment to pass to the spec
+        """
+        spec, _ = DAGSpec._auto_load(to_dag=False,
+                                     starting_dir=None,
+                                     env=env,
+                                     lazy_import=False,
+                                     reload=reload)
+        return spec
+
     @classmethod
     def from_directory(cls, path_to_dir):
-        """Construct a DAGSpec from a directory
+        """
+        Construct a DAGSpec from a directory. Product and upstream are
+        extracted from sources
 
         Parameters
         ----------
@@ -332,7 +486,9 @@ class DAGSpec(MutableMapping):
 
     @classmethod
     def from_files(cls, files):
-        """Construct DAGSpec from list of files or glob-like pattern
+        """
+        Construct DAGSpec from list of files or glob-like pattern. Product and
+        upstream are extracted from sources
 
         Parameters
         ----------
@@ -355,20 +511,23 @@ class DAGSpec(MutableMapping):
                              f'invalid extensions: {invalid}. '
                              f'Allowed extensions are: {valid_extensions}')
 
-        tasks = [{
-            'source': file_,
-            'name': str(Path(file_).with_suffix('').name)
-        } for file_ in files]
-
-        return cls({'tasks': tasks})
+        tasks = [{'source': file_} for file_ in files]
+        meta = {'extract_product': True, 'extract_upstream': True}
+        return cls({'tasks': tasks, 'meta': meta})
 
 
 class Meta:
     """Schema for meta section in pipeline.yaml
     """
     VALID = {
-        'extract_upstream', 'extract_product', 'product_default_class',
-        'product_relative_to_source', 'jupyter_hot_reload', 'source_loader'
+        'extract_upstream',
+        'extract_product',
+        'product_default_class',
+        'product_relative_to_source',
+        'jupyter_hot_reload',
+        'source_loader',
+        'jupyter_functions_as_notebooks',
+        'import_tasks_from',
     }
 
     @classmethod
@@ -393,7 +552,7 @@ class Meta:
             meta['extract_upstream'] = True
 
         if 'extract_product' not in meta:
-            meta['extract_product'] = True
+            meta['extract_product'] = False
 
         if 'product_relative_to_source' not in meta:
             meta['product_relative_to_source'] = False
@@ -401,10 +560,22 @@ class Meta:
         if 'jupyter_hot_reload' not in meta:
             meta['jupyter_hot_reload'] = False
 
+        if 'jupyter_functions_as_notebooks' not in meta:
+            meta['jupyter_functions_as_notebooks'] = False
+
+        if 'import_tasks_from' not in meta:
+            meta['import_tasks_from'] = None
+
         if 'source_loader' not in meta:
             meta['source_loader'] = None
         else:
-            meta['source_loader'] = SourceLoader(**meta['source_loader'])
+            try:
+                meta['source_loader'] = SourceLoader(**meta['source_loader'])
+            except Exception as e:
+                msg = ('Error initializing SourceLoader with '
+                       f'{meta["source_loader"]}. Error message: {e.args[0]}')
+                e.args = (msg, )
+                raise
 
         defaults = {
             'SQLDump': 'File',
@@ -421,6 +592,16 @@ class Meta:
                 if class_ not in meta['product_default_class']:
                     meta['product_default_class'][class_] = prod
 
+        # validate keys and values in product_default_class
+        for task_name, product_name in meta['product_default_class'].items():
+            try:
+                validate_task_class_name(task_name)
+                validate_product_class_name(product_name)
+            except Exception as e:
+                msg = f'Error validating product_default_class: {e.args[0]}'
+                e.args = (msg, )
+                raise
+
         return meta
 
     @classmethod
@@ -430,7 +611,7 @@ class Meta:
         return {v: None for v in cls.VALID}
 
 
-def process_tasks(dag, tasks, dag_spec, root_path=None):
+def process_tasks(dag, dag_spec, root_path=None):
     """
     Initialize Task objects from TaskSpec, extract product and dependencies
     if needed and set the dag dependencies structure
@@ -443,7 +624,7 @@ def process_tasks(dag, tasks, dag_spec, root_path=None):
     extract_prod = dag_spec['meta']['extract_product']
 
     # first pass: init tasks and them to dag
-    for task_dict in tasks:
+    for task_dict in dag_spec['tasks']:
         fn = task_dict['class']._init_source
         kwargs = {'kwargs': {}, **task_dict}
         source = call_with_dictionary(fn, kwargs=kwargs)
@@ -461,28 +642,28 @@ def process_tasks(dag, tasks, dag_spec, root_path=None):
         source_obj[task] = source
 
     # second optional pass: extract upstream
+    tasks = list(dag.values())
+
     if extract_up:
-        for task in list(dag.values()):
+        for task in tasks:
             upstream[task] = task.source.extract_upstream()
             logger.debug('Extracted upstream dependencies for task %s: %s',
                          task.name, upstream[task])
 
     # Last pass: set upstream dependencies
-    for task in list(dag.values()):
+    for task in tasks:
         if upstream[task]:
             for task_name in upstream[task]:
-                task.set_upstream(dag[task_name])
+                try:
+                    up = dag[task_name]
+                except KeyError:
+                    names = [t.name for t in tasks]
+                    raise KeyError('Error processing spec. Extracted '
+                                   'a reference to a task with name '
+                                   f'{task_name!r}, but a task with such name '
+                                   f'doesn\'t exist. Loaded tasks: {names}')
 
-
-def init_clients(dag, clients):
-    for class_name, dotted_path in clients.items():
-
-        class_ = getattr(tasks, class_name, None)
-
-        if not class_:
-            class_ = getattr(products, class_name)
-
-        dag.clients[class_] = load_dotted_path(dotted_path)()
+                task.set_upstream(up)
 
 
 def normalize_task(task):
@@ -492,3 +673,13 @@ def normalize_task(task):
         return {'source': task}
     else:
         return task
+
+
+def add_base_path_to_source_if_relative(task, base_path):
+    path = Path(task['source'])
+    relative_source = not path.is_absolute()
+
+    # must be a relative source with a valid extension, otherwise, it can
+    # be a dotted path
+    if relative_source and path.suffix in set(suffix2taskclass):
+        task['source'] = str(Path(base_path, task['source']).resolve())
