@@ -1,16 +1,31 @@
 from ploomber.tasks.tasks import (PythonCallable, ShellScript, DownloadFromURL,
                                   Link, Input, task_factory)
-from ploomber.tasks.Task import Task
 from ploomber.tasks.TaskFactory import TaskFactory
 from ploomber.tasks.sql import (SQLScript, SQLDump, SQLTransfer, SQLUpload,
                                 PostgresCopyFrom)
 from ploomber.tasks.notebook import NotebookRunner
 from ploomber.tasks.aws import UploadToS3
 from ploomber.tasks.param_forward import input_data_passer, in_memory_callable
+from ploomber.tasks.taskgroup import TaskGroup
+from ploomber.tasks.abc import Task
 
 __all__ = [
-    'PythonCallable', 'ShellScript', 'TaskFactory', 'Task', 'SQLScript',
-    'SQLDump', 'SQLTransfer', 'SQLUpload', 'PostgresCopyFrom',
-    'NotebookRunner', 'DownloadFromURL', 'Link', 'Input', 'UploadToS3',
-    'input_data_passer', 'in_memory_callable'
+    'Task',
+    'PythonCallable',
+    'ShellScript',
+    'TaskFactory',
+    'SQLScript',
+    'SQLDump',
+    'SQLTransfer',
+    'SQLUpload',
+    'PostgresCopyFrom',
+    'NotebookRunner',
+    'DownloadFromURL',
+    'Link',
+    'Input',
+    'UploadToS3',
+    'TaskGroup',
+    'input_data_passer',
+    'in_memory_callable',
+    'task_factory',
 ]
